package cn.rankin.apiweb.service.product;

import cn.rankin.apiweb.code.ApiWebCode;
import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.common.utils.api.page.Page;
import cn.rankin.data.api.app.dto.ItemSearchDTO;
import cn.rankin.data.api.app.vo.GoodsVo;
import cn.rankin.data.api.app.vo.ItemVo;
import cn.rankin.data.api.product.entity.*;
import cn.rankin.data.api.product.vo.CourseItemVo;
import cn.rankin.data.api.product.vo.SupportItemVo;
import org.springframework.cloud.netflix.feign.FeignClient;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@FeignClient(name = "${service.product.name}")
public interface ProductClient {

    @RequestMapping(value = "/recommend/courses", method = RequestMethod.GET)
    APIResult<List<Recommend>> getRecommendCourses(@RequestParam("merchantId") String merchantId);

    @RequestMapping(value = "/merchant/product/detail", method = RequestMethod.GET)
    APIResult<MerchantProduct> getMerchantProduct(@RequestParam("pid") String pid, @RequestParam("merchantId") String merchantId);

    @RequestMapping(value = "/app/item/ids", method = RequestMethod.POST)
    APIResult<List<ItemVo>> findItemByIds(@RequestParam("merchantId") String merchantId, @RequestBody List<String> productIdList);

    @RequestMapping(value = "/app/item/ids/page", method = RequestMethod.POST)
    APIResult<Page<ItemVo>> findItemPageByIds(@RequestBody ItemSearchDTO itemSearchDTO);

    @RequestMapping(value = "/app/course/{courseId}/lessons", method = RequestMethod.GET)
    APIResult<Course> getCourseLessons(@PathVariable("courseId") String courseId);

    @RequestMapping(value = "/app/course/{courseId}/detail", method = RequestMethod.GET)
    APIResult<CourseItemVo> getCourseDetail(@PathVariable("courseId") String courseId, @RequestParam("merchantId") String merchantId);

    @RequestMapping(value = "/app/course/{courseId}/supports", method = RequestMethod.GET)
    APIResult<List<ItemVo>> getSupportByCourseId(@PathVariable("courseId") String courseId, @RequestParam("merchantId") String merchantId);

    @RequestMapping(value = "/app/support/{supportId}", method = RequestMethod.GET)
    APIResult<SupportItemVo> getSupport(@PathVariable("supportId") String supportId, @RequestParam("merchantId") String merchantId);

    @RequestMapping(value = "/app/lesson/{lessonId}", method = RequestMethod.GET)
    APIResult<Lesson> getLesson(@PathVariable("lessonId") String lessonId, @RequestParam("courseId") String courseId);

    @RequestMapping(value = "/app/goods", method = RequestMethod.POST)
    APIResult<List<GoodsVo>> findGoodsByIds(@RequestBody List<String> goodsIdList, @RequestParam("merchantId") String merchantId);

    @RequestMapping(value = "/goods/{goodsId}", method = RequestMethod.GET)
    APIResult<Goods> findGoods(@PathVariable("goodsId") String goodsId);

    @Component
    class ProductClientHystrix implements ProductClient {

        @Override
        public APIResult getRecommendCourses(String merchantId) {
            return APIResult.error(ApiWebCode.SERVER_ERROR);
        }

        @Override
        public APIResult getMerchantProduct(String pid, String merchantId) {
            return APIResult.error(ApiWebCode.SERVER_ERROR);
        }

        @Override
        public APIResult findItemByIds(String merchantId, List<String> productIdList) {
            return APIResult.error(ApiWebCode.SERVER_ERROR);
        }

        @Override
        public APIResult findItemPageByIds(ItemSearchDTO itemSearchDTO) {
            return APIResult.error(ApiWebCode.SERVER_ERROR);
        }

        @Override
        public APIResult<Course> getCourseLessons(String courseId) {
            return APIResult.error(ApiWebCode.SERVER_ERROR);
        }

        @Override
        public APIResult<CourseItemVo> getCourseDetail(String courseId, String merchantId) {
            return APIResult.error(ApiWebCode.SERVER_ERROR);
        }

        @Override
        public APIResult<List<ItemVo>> getSupportByCourseId(String courseId, String merchantId) {
            return APIResult.error(ApiWebCode.SERVER_ERROR);
        }

        @Override
        public APIResult<Lesson> getLesson(String lessonId, String courseId) {
            return APIResult.error(ApiWebCode.SERVER_ERROR);
        }

        @Override
        public APIResult getSupport(String supportId, String merchantId) {
            return APIResult.error(ApiWebCode.SERVER_ERROR);
        }

        @Override
        public APIResult<List<GoodsVo>> findGoodsByIds(List<String> goodsIdList, String merchantId) {
            return APIResult.error(ApiWebCode.SERVER_ERROR);
        }

        @Override
        public APIResult<Goods> findGoods(String goodsId) {
            return APIResult.error(ApiWebCode.SERVER_ERROR);
        }
    }
}
